------------------------------------------------------
-- AdSpace.lua
-- Version 1800.3
------------------------------------------------------

FAS_SPECIAL_VENDOR_COLOR = {r=1.0, g=0.5, b=0.25};
FAS_INDENT = "  ";

-- Configuration
FAS_Config = { };
FAS_Config.Tooltip = true;
FAS_Config.ShowCost = false;

local function FAS_Tooltip_Hook(frame, name, link, source)

	if ( FAS_Config.Tooltip and link ~= nil) then
		local _, _, itemID  = string.find(link, "item:(%d+):%d+:%d+:%d+");
		itemID = tonumber(itemID);
		local itemInfo = FAS_ItemInfo[itemID];

		if (itemInfo ~= nil) then
			if (source == "MERCHANT") then
				return false; -- kinda silly to show merchants who sell this when we're seeing this at a merchant
			end
			local myFaction = UnitFactionGroup("player");
			local vendors = GFWTable.Merge(FAS_VendorInfo[myFaction][itemID], FAS_VendorInfo["Neutral"][itemID]);
			local note = itemInfo.note;
			
			local color;
			local intro;
			if (note ~= nil) then
				color = FAS_SPECIAL_VENDOR_COLOR;
			else
				color = GFW_FONT_COLOR;
			end
			if (FAS_Config.ShowCost) then
				intro = string.format(SOLD_FOR_PRICE_BY, GFWUtils.TextGSC(itemInfo.b));
			else
				intro = SOLD_BY;
			end
					
			if (table.getn(vendors) > 1) then
				frame:AddLine(intro..":", color.r, color.g, color.b);
				for i, aVendor in vendors do
					local vendorLoc = FAS_VendorLocations[aVendor];
					if (vendorLoc ~= nil) then
						local vendorName = FAS_Localized[aVendor] or aVendor;
						local vendorLocation = FAS_Localized[vendorLoc] or vendorLoc;
						frame:AddLine(FAS_INDENT..string.format(VENDOR_LOCATION_FORMAT, vendorName, vendorLocation), color.r, color.g, color.b);
					else
						GFWUtils.Print(GFWUtils.Red("AdSpace error: ").."Can't find location for "..aVendor..". Please report to gazmik@fizzwidget.com.");
						return false;
					end
				end
			elseif (table.getn(vendors) == 1) then
				local vendorLoc = FAS_VendorLocations[vendors[1]];
				if (vendorLoc ~= nil) then
					local vendorName = FAS_Localized[vendors[1]] or vendors[1];
					local vendorLocation = FAS_Localized[vendorLoc] or vendorLoc;
					frame:AddLine(intro.." "..string.format(VENDOR_LOCATION_FORMAT, vendorName, vendorLocation), color.r, color.g, color.b);
				else
					GFWUtils.Print(GFWUtils.Red("AdSpace error: ").."Can't find location for "..vendorName..". Please report to gazmik@fizzwidget.com.");
					return false;
				end
			else
				local found = false;
				for _, faction in {"Alliance", "Horde", "Neutral"} do
					if (FAS_VendorInfo[faction][itemID] ~= nil) then
						found = true;
					end
				end
				if not (found) then
					GFWUtils.Print(GFWUtils.Red("AdSpace error: ")..link.."("..itemID..") is listed but has no vendors. Please report to gazmik@fizzwidget.com.");
					return false;
				end
			end
			if (note ~= "") then
				frame:AddLine(note, color.r, color.g, color.b);
			end
			return true;
		end
		
		local libramInfo = FAS_LibramInfo[itemID];
		if (libramInfo ~= nil) then
			local color = FAS_SPECIAL_VENDOR_COLOR;
			local returnToName = FAS_Localized[libramInfo.name] or libramInfo.name;
			local returnToLocation = FAS_Localized[FAS_VendorLocations[libramInfo.name]] or FAS_VendorLocations[libramInfo.name];
			local bonus = FAS_Localized[libramInfo.bonus] or libramInfo.bonus;
			frame:AddLine(RETURN_TO.." "..string.format(VENDOR_LOCATION_FORMAT, returnToName, returnToLocation), color.r, color.g, color.b);
			frame:AddLine(string.format(ARCANUM_FORMAT, bonus), color.r, color.g, color.b);
			return true;
		end

		local otherInfo = FAS_OtherItemInfo[itemID];
		if (otherInfo ~= nil) then
			local color = FAS_SPECIAL_VENDOR_COLOR;
			frame:AddLine(otherInfo, color.r, color.g, color.b);
			return true;
		end
	end
	
end

function FAS_OnLoad()

	-- Register Slash Commands
	SLASH_FAS1 = "/adspace";
	SLASH_FAS2 = "/ads";
	SlashCmdList["FAS"] = function(msg)
		FAS_ChatCommandHandler(msg);
	end
	
	-- Hook in new tooltip code
	--FAS_Old_Tooltip_Hook = GFWTooltip_AddTooltipInfo;
	--GFWTooltip_AddTooltipInfo = FAS_Tooltip_Hook;
	GFWTooltip_AddCallback("GFW_AdSpace", FAS_Tooltip_Hook);

	GFWUtils.Print("Fizzwidget AdSpace initialized!");
end

function FAS_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print("Fizzwidget AdSpace:");
		GFWUtils.Print("/adspace (or /ads)");
		GFWUtils.Print("- "..GFWUtils.Hilite("help").." - Print this helplist.");
		GFWUtils.Print("- "..GFWUtils.Hilite("status").." - Check current settings.");
		GFWUtils.Print("- "..GFWUtils.Hilite("tooltip on").." | "..GFWUtils.Hilite("off").." - Enable/disable display of vendor availability in recipe tooltips.");
		GFWUtils.Print("- "..GFWUtils.Hilite("showcost on").." | "..GFWUtils.Hilite("off").." - Also show vendor prices for recipes.");
		GFWUtils.Print("- "..GFWUtils.Hilite("info <item link>").." - Show vendor info for an item in the chat window.");
		return;
	end
	
	if (msg == "tooltip on") then
		FAS_Config.Tooltip = true;
		if ( FAS_Config.ShowCost ) then
			GFWUtils.Print("Showing vendor availability (or related info) and pricing in tooltips for select items.");
		else
			GFWUtils.Print("Only showing vendor availability (or related info) in tooltips for select items.");
		end
		return;
	end
	if (msg == "tooltip off") then
		FAS_Config.Tooltip = false;
		GFWUtils.Print(GFWUtils.Hilite("Not").." showing extra info in item tooltips.");
		return;
	end
	
	if (msg == "showcost on") then
		FAS_Config.ShowCost = true;
		GFWUtils.Print("Showing vendor availability (or related info) and pricing in tooltips for select items.");
		if not ( FAS_Config.Tooltip ) then
			GFWUtils.Print("(or would, if tooltip modification was enabled. Type "..GFWUtils.Hilite("/ads tooltip on").." to do so.)");
		end
		return;
	end
	if (msg == "showcost off") then
		FAS_Config.ShowCost = false;
		if ( FAS_Config.Tooltip ) then
			GFWUtils.Print("Only showing vendor availability (or related info) in tooltips for select items.");
		else
			GFWUtils.Print(GFWUtils.Hilite("Not").." showing extra info in item tooltips.");
		end
		return;
	end
	
	if ( msg == "status" ) then
		if ( FAS_Config.Tooltip ) then
			if ( FAS_Config.ShowCost ) then
				GFWUtils.Print("Showing vendor availability (or related info) and pricing in tooltips for select items.");
			else
				GFWUtils.Print("Only showing vendor availability (or related info) in tooltips for select items.");
			end
		else
			GFWUtils.Print(GFWUtils.Hilite("Not").." showing extra info in item tooltips.");
		end
		return;
	end
	
	if (msg == "test") then
		local itemInfoCount = 0;
		for itemID in FAS_ItemInfo do
			local found = false;
			for _, faction in {"Alliance", "Horde", "Neutral"} do
				if (FAS_VendorInfo[faction][itemID] ~= nil) then
					found = true;
				end
			end
			if not (found) then
				GFWUtils.Print("Item ID "..itemID.." not found in FAS_VendorInfo.");
			end
			itemInfoCount = itemInfoCount + 1;
		end
		GFWUtils.Print(itemInfoCount.." entries in FAS_ItemInfo.");
		for _, faction in {"Alliance", "Horde", "Neutral"} do
			local vendorInfoCount = 0;
			for itemID in FAS_VendorInfo[faction] do
				if (FAS_ItemInfo[itemID] == nil) then
					GFWUtils.Print("Item ID "..itemID.." not found in FAS_ItemInfo.");
				end
				vendorInfoCount = vendorInfoCount + 1;
			end
			GFWUtils.Print(vendorInfoCount.." entries in FAS_VendorInfo["..faction.."].");
		end
		return;
	end

	local _, _, cmd, itemLink = string.find(msg, "(%w+) (.+)");
	if (cmd == "vendor" or cmd == "info") then
		local _, _, itemID  = string.find(itemLink, ".Hitem:(%d+):%d+:%d+:%d+.h%[[^]]+%].h");
		if (itemID == nil or itemID == "") then
			GFWUtils.Print("Usage: "..GFWUtils.Hilite("/ads info <item link>"));
			return;
		end
		itemID = tonumber(itemID);
		
		local itemInfo = FAS_ItemInfo[itemID];
		if (itemInfo ~= nil) then
			local myFaction = UnitFactionGroup("player");
			local vendors = GFWTable.Merge(FAS_VendorInfo[myFaction][itemID], FAS_VendorInfo["Neutral"][itemID]);
			local note = itemInfo.note;
			local intro = string.format(SOLD_FOR_PRICE_BY, GFWUtils.TextGSC(itemInfo.b));

			if (vendors == nil or vendors == {}) then
				GFWUtils.Print(GFWUtils.Red("AdSpace error: ")..itemLink.."("..itemID..") is listed but has no vendors. Please report to gazmik@fizzwidget.com.");
				return;
			end
			GFWUtils.Print(intro);
			for i, aVendor in vendors do
				local vendorName = FAS_Localized[aVendor] or aVendor;
				local vendorLocation = FAS_Localized[FAS_VendorLocations[aVendor]] or FAS_VendorLocations[aVendor];
				GFWUtils.Print(string.format(VENDOR_LOCATION_FORMAT, vendorName, vendorLocation));
			end
			if (note ~= nil and note ~= "") then
				GFWUtils.Print(GFWUtils.Hilite(note));
			end
			return;
		end
		
		local libramInfo = FAS_LibramInfo[itemID];
		if (libramInfo ~= nil) then
			local returnToName = FAS_Localized[libramInfo.name] or libramInfo.name;
			local returnToLocation = FAS_Localized[FAS_VendorLocations[libramInfo.name]] or FAS_VendorLocations[libramInfo.name];
			local bonus = FAS_Localized[libramInfo.bonus] or libramInfo.bonus;
			GFWUtils.Print(RETURN_TO.." "..string.format(VENDOR_LOCATION_FORMAT, returnToName, returnToLocation));
			GFWUtils.Print(bonus);
			return;
		end
		
		local otherInfo = FAS_OtherItemInfo[itemID];
		if (otherInfo ~= nil) then
			GFWUtils.Print(otherInfo);
			return;
		end

		GFWUtils.Print("Nothing known about "..itemLink..".");
		return;
	end
	
	-- if we made it down here, there were args we didn't understand... time to remind the user what to do.
	FAS_ChatCommandHandler("help");

end

function FAS_CheckMerchant(itemID)
	for merchantIndex = 1, GetMerchantNumItems() do
		local link = GetMerchantItemLink(merchantIndex);
		local _, _, merchantItemID  = string.find(link, "item:(%d+):%d+:%d+:%d+");
		if (tonumber(merchantItemID) == itemID) then
			return true;
		end
	end
	return false;
end

function FAS_Translate(langCode)

	tempTranslations = {};
	
	local localizedVendorInfo = getglobal("FAS_VendorInfo_"..langCode);
	local localizedVendorLocations = getglobal("FAS_VendorLocations_"..langCode);
	for faction, factionVendorList in FAS_VendorInfo do
		for itemID, vendorList in factionVendorList do
			local localizedVendors = localizedVendorInfo[faction][itemID];
			if (localizedVendors ~= nil and type(localizedVendors) == "table") then
				for index, name in vendorList do
					local localizedName = localizedVendors[index];
					
					if (localizedName == nil) then break; end
					
					if (localizedName ~= name) then
						if (tempTranslations[name] == nil) then
							tempTranslations[name] = {};
						end
						table.insert(tempTranslations[name], localizedName);
					end
					
					local location = FAS_VendorLocations[name];
					local localizedLocation = localizedVendorLocations[localizedName];
					if (localizedLocation ~= nil and localizedLocation ~= location) then
						if (tempTranslations[location] == nil) then
							tempTranslations[location] = {};
						end
						table.insert(tempTranslations[location], localizedLocation);
					end
				end
			end
		end
	end
	
	local localizedLibramInfo = getglobal("FAS_LibramInfo_"..langCode);
	for itemID, libramInfo in FAS_LibramInfo do
		local localizedInfo = localizedLibramInfo[itemID];
		if (localizedInfo ~= nil and type(localizedInfo) == "table") then
					
			if (localizedInfo.name ~= libramInfo.name) then
				if (tempTranslations[libramInfo.name] == nil) then
					tempTranslations[libramInfo.name] = {};
				end
				table.insert(tempTranslations[libramInfo.name], localizedInfo.name);
			end
			if (localizedInfo.bonus ~= libramInfo.bonus) then
				if (tempTranslations[libramInfo.bonus] == nil) then
					tempTranslations[libramInfo.bonus] = {};
				end
				table.insert(tempTranslations[libramInfo.bonus], localizedInfo.bonus);
			end
			
			local location = FAS_VendorLocations[libramInfo.name];
			local localizedLocation = localizedVendorLocations[localizedInfo.name];
			if (localizedLocation ~= nil and localizedLocation ~= location) then
				if (tempTranslations[location] == nil) then
					tempTranslations[location] = {};
				end
				table.insert(tempTranslations[location], localizedLocation);
			end
		end
	end
	
	FAS_Config[langCode] = {};
	for baseString, translations in tempTranslations do
		if (table.getn(translations) == 1) then
			FAS_Config[langCode][baseString] = translations[1];
		else
			local mergedTranslations = {}
			for _, translation in translations do
				if (GFWTable.KeyOf(mergedTranslations, translation) == nil) then
					table.insert(mergedTranslations, translation);
				end
			end
			if (table.getn(mergedTranslations) == 1) then
				FAS_Config[langCode][baseString] = mergedTranslations[1];
			else
				FAS_Config[langCode][baseString] = mergedTranslations;
			end
		end
	end

end
------------------------------------------------------
-- Runtime loading
------------------------------------------------------

FAS_OnLoad();
